'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''

#stdlib
import time
import math
#lib
import pygame
from pygame.locals import *
#custom
import data
from util import * # ja..

SCREENWIDTH = 800
SCREENHEIGHT = 600



class Actor:
	def __init__(self,x,y):
		self.position = Vector(x,y)

		self.ropeStatus = "none" # none, shoot, fixed

		self.ropeDirection = Vector(0,0) # set, when we are currently shooting a rope
		self.ropeShootSpeed = 1000.0 # pixels per sec
		self.ropeLength = 0 # set to something, when rope is shooting / finished
		self.ropeHeadPosition = None # set to smth, when rope is finished

		self.shortenRope = 0

		self.moveVector  = Vector(0,30)
		self.rotationSpeed = 0.0 # winkelgrad per sec

		self.groundObstacle = None # if we are standing on obstacles,  here it is

		self.keyMoveX = 0
		self.keyJump = 0


	def draw(self,window):
		"""
		draw actor and rope
		"""
		# draw actor
		pygame.draw.circle(window,(0,255,0), (self.position.x,self.position.y), 5, 0)
		# draw rope
		if self.ropeStatus != "none":
			ropeHeadPosition = self.getRopeHeadPosition()
			pygame.draw.line(window,(50,255,50),(self.position.x,self.position.y),
						(ropeHeadPosition.x,ropeHeadPosition.y),2)
		return 1

	def getRopeHeadPosition(self):
		"""
		where is the rope head at?
		"""
		if self.ropeHeadPosition is not None:
			return self.ropeHeadPosition
		else:
			return ( self.position + ( self.ropeDirection  * self.ropeLength) )

	def update(self,timeElapsed):
		"""
		update actor position
		"""
		# if shooting, ropelength++
		if self.groundObstacle is None:
			if self.ropeStatus == "shoot":
				self.ropeLength += ( self.ropeShootSpeed * timeElapsed)
				self.moveGravity(timeElapsed)
			elif self.ropeStatus == "fixed":
				self.moveOnRope(timeElapsed)
			elif self.ropeStatus == "none":
				self.moveGravity(timeElapsed)

			if self.ropeStatus == "fixed" and self.shortenRope != 0:
				# shorten rope and recalc pos according to it
				shortenBy = 10.0 * self.shortenRope
				self.ropeLength = shortenBy # not sure if we need to shorten length?
				ropeHead = self.getRopeHeadPosition()
				towardRopeHead = Vector(self.position,ropeHead).unit()
				self.position += towardRopeHead * shortenBy
		else:
			# we are standing on an obstacles
			self.moveKeys(timeElapsed)


	def moveKeys(self,timeElapsed):
		"""
		move with wasd
		"""
		self.position += Vector(60 * timeElapsed * self.keyMoveX,0)

		# check wether we are falling of the obstacle
		if (( self.position.x <= self.groundObstacle.rect.x) or
			( self.position.x >= self.groundObstacle.rect.x + self.groundObstacle.rect.width) ):
			self.groundObstacle = None


	def moveGravity(self,timeElapsed):
		"""
		movement in free fall
		"""
		self.moveVector += Vector(0,150*timeElapsed) #* timeElapsed
		self.position += (self.moveVector * timeElapsed)

	def moveOnRope(self,timeElapsed):
		"""
		position updates if we are on a rope
		"""

		# if not rotation speed going left or right?
		initialSpeed = 10.0
		# we change the rotationspeed if:
		# -- it is 0
		# -- or we are moving up a slope and are getting to slow => swing back
		ropeVector = self.getRopeHeadPosition() - self.position
		ropeAngle = ropeVector.angle_in_degrees( Vector(0,-1))

		if ((abs(self.rotationSpeed) <= 0.0001) or
				(
					(abs(ropeAngle/self.rotationSpeed) >= 1.4) and
						(
						(self.rotationSpeed <= 0.0 and self.position.x > self.getRopeHeadPosition().x) or
						(self.rotationSpeed >= 0.0 and self.position.x < self.getRopeHeadPosition().x)
						)
				)
			):
			self.rotationSpeed *= -0.4
			if self.rotationSpeed == 0.0:
				self.rotationSpeed = 10
		ropeVector = ropeVector.rotate(math.radians(self.rotationSpeed)*timeElapsed)
		self.position = self.getRopeHeadPosition() - ropeVector

		slowDown = 70 * timeElapsed
		speedUp = 100 * timeElapsed
		# get slower, if we are already on other side of rope
		if self.rotationSpeed < 0.0:
			if (self.position.x > self.getRopeHeadPosition().x):
				self.rotationSpeed += slowDown # get slower
			else:
				# get faster
				self.rotationSpeed -= speedUp
		elif self.rotationSpeed > 0.0:
			if self.position.x < self.getRopeHeadPosition().x:
				self.rotationSpeed -= slowDown
			else:
				#get faster
				self.rotationSpeed += speedUp

		#print self.rotationSpeed


	def keyUp(self,key):
		if key == K_a or key == K_d:
			self.keyMoveX = 0
		elif key in [K_w,K_s]:
			self.shortenRope = 0

	def keyDown(self,key):
		if key == K_a:
			self.keyMoveX = -1
		elif key == K_d:
			self.keyMoveX = 1
		elif key == K_w:
			self.shortenRope = 1
		elif key == K_s:
			self.shortenRope = -1

	def dropRopeRequest(self):
		"""
		player lets loose of the rope
		"""
		if self.ropeStatus == "fixed":
			self.aquireRotationMoveVector()
			self.rotationSpeed = 0
			self.ropeStatus = "none"

	def aquireRotationMoveVector(self):
		"""
		when we let go of the rope, need get
		current move vector
		"""
		ropeHead = self.getRopeHeadPosition()
		ropeVector = self.getRopeHeadPosition() - self.position
		# perpindicular vector
		if self.position.x > ropeHead.x:
			self.moveVector = ropeVector.perpindicular().unit() * abs(self.rotationSpeed)
		else:
			self.moveVector = ropeVector.perpindicular().unit() * abs(self.rotationSpeed)
			self.moveVector = Vector(self.moveVector.x * -1,self.moveVector.y)


	def landedOnObstacle(self,o):
		self.groundObstacle = o
		self.moveVector = Vector(0,0)

	def shootRopeRequest(self,mousepos):
		"""
		player shoots a rope in mouse direction
		"""
		# take move vec from current perp vector
		if self.ropeStatus == "fixed":
			self.aquireRotationMoveVector()

		# move a bit up, since we dont want to be standing on ground
		self.position.y += -5
		self.groundObstacle = None
		self.ropeStatus = "shoot"
		ropeDirection = Vector( self.position,mousepos).unit()
		self.ropeHeadPosition = None
		self.ropeDirection = ropeDirection
		self.ropeLength = 2

	def ropeCollideUpdate(self,collidepos):
		"""
		shot rope collided with smth at collidepos, so we
		can now svae the ropeHeadPos. this also fixes the length
		"""
		self.ropeStatus = "fixed"
		self.ropeHeadPosition = collidepos
		self.ropeDirection = None
		self.moveVector = Vector(0,0)


class Obstacle:
	def __init__(self,x,y,w,h):
		self.position = Vector(x,y)
		self.w = w
		self.h = h
		self.rect = pygame.Rect( x,y,self.w,self.h)

	def draw(self,window):
		pygame.draw.rect(window,(255,0,0),self.rect,0)
		return 1

def main():
	print "Hello from your game's main()"
	noa = NotAnObject()
	noa.gameLoop()

class NotAnObject:
	def __init__(self):
		pygame.init() # magic
		self.window = pygame.display.set_mode( (SCREENWIDTH,SCREENHEIGHT)) # get a surface to draw on
		pygame.display.set_caption( 'simono a game' ) # set a window caption
		self.backgroundSrf  = pygame.Surface( self.window.get_size() ) # create a surface, same size as window
		self.backgroundSrf.fill( (114,167,211) ) # fill surface with black color
		self.window.blit(self.backgroundSrf, (0,0) ) # blit (=put) background on window
		self.gameFont=pygame.font.SysFont(pygame.font.get_default_font(), 30)
		pygame.display.flip() # magic, needed to display window updates

		self.running = 1
		self.lastFrametime = 0
		self.fps = 60.0

		self.actor = Actor(50,20)

		self.obstacles = []
		self.obstacles.append( Obstacle(10,10,500,20) )
		self.obstacles.append( Obstacle(80,50,60,20) )
		self.obstacles.append( Obstacle(200,100,150,20) )
		self.obstacles.append( Obstacle(400,100,150,20) )

		self.obstacles.append( Obstacle(70,200,150,20) )
		self.obstacles.append( Obstacle(500,250,150,20) )

		self.obstacles.append( Obstacle(250,350,150,20) )
		self.obstacles.append( Obstacle(600,300,150,20) )

		self.obstacles.append( Obstacle(20,450,150,20) )

	def gameLoop(self):


		testStart = Vector(300,300)
		testVec = Vector(5,50)
		self.lastFrametime = time.time()
		while self.running == 1:
			fTime = time.time()
			timeElapsed = fTime - self.lastFrametime
			if timeElapsed > 1/self.fps:
				self.lastFrametime = fTime
				# clear screen
				self.window.blit(self.backgroundSrf, (0,0) ) # blit (=put) background on window
				self.handleKbdMouse()
				# update
				self.actor.update(timeElapsed)
				# check if we rope hit smth
				if self.actor.ropeStatus == "shoot":
					ropeHeadPosition = self.actor.getRopeHeadPosition()
					ropeHeadRect = pygame.Rect(ropeHeadPosition.x,
												ropeHeadPosition.y,
												5,5)
					for o in self.obstacles:
						if o.rect.colliderect( ropeHeadRect) == True:
							self.actor.ropeCollideUpdate( ropeHeadPosition)
							break
				elif self.actor.ropeStatus == "none":
					# check wether actor landed ON an obstacle
					actorRect = pygame.Rect(self.actor.position.x,self.actor.position.y,5,5)
					for o in self.obstacles:
						topRect = pygame.Rect(o.rect.x,o.rect.y,o.rect.width,2)
						if actorRect.colliderect(topRect):
							self.actor.landedOnObstacle(o)
							break

				# draw stuff, etc
				for o in self.obstacles:
					o.draw(self.window)
				self.actor.draw(self.window)

				# basic help
				fontSrf = self.gameFont.render("RMB: shoot rope | SPACE: drop rope | W/S: shorten/lengthen rope",1,(0,0,0))
				self.window.blit(fontSrf,(50,560))
				fontSrf = self.gameFont.render("A / D: move on plattform | ENTER: restart ",1,(0,0,0))
				self.window.blit(fontSrf,(50,580))

				pygame.display.flip()

				# auto reset, if actor is out of screen
				if self.actor.position.y > SCREENHEIGHT:
					self.reset()

	def reset(self):
		self.actor = Actor(20,20)


	def handleKbdMouse(self):
		"""
		handle keyboard and mouse events
		this includes creating nets in the self.nets list when necessary
		## see http://www.pygame.org/docs/ref/event.html
		"""
		for event in pygame.event.get():
			if event.type == QUIT: # user closed the window
				self.running = 0 # stop game loop
			elif event.type == KEYDOWN:
				if event.key == K_ESCAPE:
					self.running = 0 # stop game loop
				elif event.key == K_SPACE:
					self.actor.dropRopeRequest()
				elif event.key == K_RETURN:
					self.reset()
				elif event.key in (K_w,K_a,K_s,K_d):
					self.actor.keyDown(event.key)
			elif event.type == KEYUP:
				if event.key == K_SPACE:
					self.actor.shortenRope = 0
				elif event.key in (K_w,K_a,K_s,K_d):
					self.actor.keyUp(event.key)
			elif event.type == MOUSEBUTTONDOWN:
				pass
			elif event.type == MOUSEBUTTONUP:
				# but3 == shoot the rope
				if event.button == 3:
					self.actor.shootRopeRequest(event.pos)


